use utf8;
use strict;
use warnings FATAL => 'all';

use Encode;

use Xyzzy;
use Xyzzy::Header;
use Xyzzy::Request;

sub concat {
	return undef if grep !defined, @_;
	return join('/', @_);
}

$0 = "xyzzy (initializing)";

my $file = $ARGV[0];

die "Usage: xyzzy <configfile>\n"
	unless defined $file;

my $main = new Xyzzy;
$main->include($file);
my $class = ref $main;

$0 = "$class (initializing)";

while(my $req = new Xyzzy::Request) {
	$0 = "$class (running)";

	my $oldout = *STDOUT;
	*STDOUT = *STDERR;
	my @output;

	eval {
		$main->generate($req);

		my $status = $req->status;
		my $content = $req->content;

		unless(defined $content) {
			$req->mimetype('text/html', charset => 'UTF-8');
			$content = $status->html;
		}

		$content = Encode::encode_utf8($content)
			if Encode::is_utf8($content);

		$req->setheader('Content-Length', length($content));

		push @output, "Status: $status\r\n";
		my $headers = $req->outheaders;
		while(my ($key, $vals) = each(%$headers)) {
			my $concat = 1;
			if(@$vals > 1) {
				foreach my $val (@$vals) {
					$concat = 0
						unless Xyzzy::Header::istoken($val);
				}
			}
			if($concat) {
				push @output, $key, ': ', join(', ', @$vals), "\r\n";
			} else {
				foreach my $val (@$vals) {
					push @output, $key, ': ', $val, "\r\n";
				}
			}
		}
		push @output, "\r\n", $content;
	};
	if($@) {
		warn $@;
		my $status = new Xyzzy::Status(500);
		@output = $status->response;
	}

	$0 = "$class (output)";

	foreach(@output) {
		syswrite($oldout, $_)
			or die "Writing output: $!\n";
	}
	*STDOUT = $oldout;

	$0 = "$class (idle)";
}
